#version 330 core

out vec4 FragColor[2];

in vec2 UV;

uniform sampler2D gPosition;
uniform sampler2D gNormal;
uniform sampler2D gColorSpec;

//pointlight shadow (middle light)
uniform samplerCube cubemap_tex;
uniform vec4 WS_pos_light;
uniform vec2 near_far; 

struct Light {
    vec3 Position;
    vec3 Color;
};

uniform int NR_LIGHTS;
uniform Light lights[50];
uniform vec3 viewPos;

void main()
{
	vec3 FragPos = texture(gPosition, UV).rgb;
    vec3 Normal = texture(gNormal, UV).rgb;
    vec3 Diffuse = texture(gColorSpec, UV).rgb;

	//pointlight calculations: (what is in shadow)
	float near = near_far.x;
	float far = near_far.y;
	// Calculate vector from surface point to light position:
	vec3 cm_lookup_vec = FragPos.xyz - WS_pos_light.xyz;
	// Read depth value from cubemap shadow map:
	float d_L = texture(cubemap_tex, cm_lookup_vec).r;
	// Distance between light and fragment:
	float d_S = length(cm_lookup_vec);
	// Undo previous [0;1]-mapping. Transform to world space [near,far]:
	d_L = d_L * (far - near) + near;
	// Add a small offset to prevent shadow-acne (Adjust for your scene!)
	const float eps = 0.1;

	float lightFromPointlightMiddle = 0.0;
	if (d_L + eps < d_S) {
		lightFromPointlightMiddle = 0.001;
	}
	else {
		lightFromPointlightMiddle = 0.1;
	}

    /////////////////////////////////////////////////////
    vec3 lighting  = Diffuse * lightFromPointlightMiddle;
    vec3 viewDir  = normalize(viewPos - FragPos);
    for(int i = 0; i < NR_LIGHTS; ++i)
    {
        // diffuse
        vec3 lightDir = normalize(lights[i].Position - FragPos);
        vec3 diffuse = max(dot(Normal, lightDir), 0.0) * Diffuse * lights[i].Color;
        // specular
        vec3 halfwayDir = normalize(lightDir + viewDir);  
        float spec = pow(max(dot(Normal, halfwayDir), 0.0), 16.0);
        vec3 specular = lights[i].Color * spec;
        // attenuation
        float distance = length(lights[i].Position - FragPos);
        float attenuation = 1.0 / (1.0 + 9.7 * distance + 11.8 * distance * distance);
        diffuse *= attenuation;
        specular *= attenuation;
        lighting += diffuse + specular;        
    }
    FragColor[0] = vec4(lighting, 1.0);
	FragColor[1] = vec4(0.0f,0.0f,0.0f, 1.0);
	//FragColor = vec4(lightFromPointlightMiddle, lightFromPointlightMiddle, lightFromPointlightMiddle, 1.0);
}