#version 330 core
layout (location = 0) out vec3 gPosition;
layout (location = 1) out vec3 gNormal;
layout (location = 2) out vec4 gColorSpec;

in vec2 TexCoords;
in vec3 FragPos;
in vec3 Normal;
in mat3 TBN;
in vec3 TangentViewPos;
in vec3 TangentFragPos;

uniform sampler2D DiffuseTextureSampler;
uniform sampler2D NormalTextureSampler;
uniform sampler2D HeightTextureSampler;
uniform vec3 colorOfObject;

vec2 ParallaxMapping(vec2 texCoords, vec3 viewDir)
{ 
    float height =  texture(HeightTextureSampler, texCoords).r;    
	float height_scale = 0.1;
    vec2 p = viewDir.xy / viewDir.z * (height * height_scale);
    return texCoords - p;    
} 

void main()
{    
    gPosition = FragPos;

	//parallaxMapping
	vec3 viewDir = normalize(TangentViewPos - TangentFragPos);
    
    vec2 texCoords = ParallaxMapping(TexCoords,  viewDir);       
    //if(texCoords.x > 1.0 || texCoords.y > 1.0 || texCoords.x < 0.0 || texCoords.y < 0.0)
    //    discard;


	vec3 normal_fromTex = texture(NormalTextureSampler, texCoords).rgb;
    normal_fromTex = normalize(normal_fromTex * 2.0 - 1.0);

	vec3 normal = normalize(TBN * normal_fromTex); 

    gNormal = normalize(normal);

	if(colorOfObject.xyz == 0) {
		gColorSpec.rgb = texture(DiffuseTextureSampler, texCoords).rgb;
	} else {
		gColorSpec.rgb = colorOfObject;
	}

    gColorSpec.a = texture(NormalTextureSampler, TexCoords).r;
}