#version 330 core
layout (location = 0) in vec3 position;
layout (location = 1) in vec3 normal;
layout (location = 2) in vec2 uv;
layout (location = 3) in vec3 tangent;
layout (location = 4) in vec3 bitangent;

out vec3 FragPos;
out vec2 TexCoords;
out vec3 Normal;
out mat3 TBN;
out vec3 TangentViewPos;
out vec3 TangentFragPos;

uniform mat4 M;
uniform mat4 V;
uniform mat4 P;
uniform vec3 viewPos;

void main()
{
    vec4 worldPos = M * vec4(position, 1.0);
    FragPos = worldPos.xyz; 
    TexCoords = uv;
    
    mat3 normalMatrix = transpose(inverse(mat3(M)));
    Normal = normalMatrix * normal;

	normalMatrix = transpose(inverse(mat3(M)));
    vec3 T = normalize(normalMatrix * -tangent);
    vec3 B = normalize(normalMatrix * bitangent);
    vec3 N = normalize(normalMatrix * normal); 

	TBN = mat3(T, B, N);
	mat3 tTBN = transpose(TBN);

	TangentViewPos  = tTBN * viewPos;
    TangentFragPos  = tTBN * FragPos;

    gl_Position = P*V* worldPos;
}