#version 330 core

in vec2 UV;
in vec3 positionWorld;

in vec3 lightPosTangent;
in vec3 viewPosTangent;
in vec3 vertexPosTangent;

out vec4 outColor[2];

uniform sampler2D DiffuseTextureSampler;
uniform sampler2D NormalTextureSampler;
uniform mat4 V;
uniform mat4 M;
uniform mat4 P;
uniform vec3 lightPosWorld;

void main(){

    vec3 normal = texture(NormalTextureSampler, UV).rgb;
    normal = normalize(normal * 2.0 - 1.0);

    vec3 color = texture(DiffuseTextureSampler, UV).rgb;
    vec3 ambient = 0.35 * color;
    vec3 lightDir = normalize(lightPosTangent - vertexPosTangent);
    float diff = max(dot(lightDir, normal), 0.0);
    vec3 diffuse = diff * color * 0.6;
    vec3 viewDir = normalize(viewPosTangent - vertexPosTangent);
    vec3 reflectDir = reflect(-lightDir, normal);
    vec3 halfwayDir = normalize(lightDir + viewDir);  
    float spec = pow(max(dot(normal, halfwayDir), 0.0), 5.0);
    vec3 specular = vec3(0.2) * spec;
    
    outColor[0] = vec4(ambient + diffuse +  specular, 1.0f);
	//outColor[0] = vec4(1.0f, 1.0f, 1.0f, 1.0f);
	outColor[1] = vec4(0.0,0.0,0.0,0.0);
}